#!/bin/bash
#
# System startup script for service Hmedia PrintServer
#
#
# /etc/init.d/printserver
#   and its symbolic link
# /usr/sbin/printserver
#
# chkconfig: 345 99 00
# description: Hmedia PrintServer providing print functionality for iNEWS
# 
### BEGIN INIT INFO
# Provides:          PrintServer
# Required-Start:    $syslog $local_fs $network 
# Should-Start: $time sendmail
# Required-Stop:     $syslog $local_fs $network
# Should-Stop: $time sendmail
# Default-Start:     3 5
# Default-Stop:      0 1 2 6
# Short-Description: Hmedia PrintServer providing print functionality for iNEWS
# Description:       Start Hmedia PrintServer to allow PrintClients to connect
#	to this machine and print iNEWS content
#	The PrintServer connects the different iNEWS machines via FTP.
#	The PrintClients also use the FTP protocol to talk with PrintServer.
#	Therefore the vsftp server must run.
#	Because the PrintServer is a Java program the JRT >= 1.5 has to run.
### END INIT INFO

# NOTE:
# Return values acc. to LSB for all commands but status:#!/bin/bash
#
# System startup script for service Hmedia PrintServer
#
#
# /etc/init.d/printserver
#   and its symbolic link
# /usr/sbin/printserver
#
# chkconfig: 345 99 00
# description: Hmedia PrintServer providing print functionality for iNEWS
# 
### BEGIN INIT INFO
# Provides:          PrintServer
# Required-Start:    $syslog $local_fs $network 
# Should-Start: $time sendmail
# Required-Stop:     $syslog $local_fs $network
# Should-Stop: $time sendmail
# Default-Start:     3 5
# Default-Stop:      0 1 2 6
# Short-Description: Hmedia PrintServer providing print functionality for iNEWS
# Description:       Start Hmedia PrintServer to allow PrintClients to connect
#	to this machine and print iNEWS content
#	The PrintServer connects the different iNEWS machines via FTP.
#	The PrintClients also use the FTP protocol to talk with PrintServer.
#	Therefore the vsftp server must run.
#	Because the PrintServer is a Java program the JRT >= 1.5 has to run.
### END INIT INFO

# NOTE:
# Return values acc. to LSB for all commands but status:
# 0	  - success
# 1       - generic or unspecified error
# 2       - invalid or excess argument(s)
# 3       - unimplemented feature (e.g. "reload")
# 4       - user had insufficient privileges
# 5       - program is not installed
# 6       - program is not configured
# 7       - program is not running
# 8--199  - reserved (8--99 LSB, 100--149 distrib, 150--199 appl)
# 
# Note that starting an already running service, stopping
# or restarting a not-running service as well as the restart
# with force-reload (in case signaling is not supported) are
# considered a success.

# Setup initial variables
# =======================
# PrintServer assumes to find the java command in /usr/bin. That's true for
# correct java installations. But maybe you have a special system...
JAVA_HOME="/usr"


# Functions
running_instances() {
	ps -ef | grep "printserver.jar" | grep -v "grep" | wc -l | sed -e 's/^[\t ]*//'
}

# Set the directory where the PrintServer software is sitting in:
if [ "$(readlink $0)" != "" ]
then
        PRINTSERVER_HOME="$(dirname $(readlink $0))"
elif [ "$(dirname $0)" == "." ]
then
        PRINTSERVER_HOME="$(pwd)"
elif [ "$(dirname $0)" == ".." ]
then
        PRINTSERVER_HOME="$(dirname $(pwd))"
elif [ "$(dirname $0 | cut -c1)" == "/" ]
then
	PRINTSERVER_HOME="$(dirname $0)"
elif [ "$(pwd)" == "/" ]
then
        PRINTSERVER_HOME="/$(dirname $0)"
else
        PRINTSERVER_HOME="$(pwd)/$(dirname $0)"
fi

# Check for existence of needed config files
PRINTSERVER_CONFIG="$PRINTSERVER_HOME/conf.xml"
test -r "$PRINTSERVER_CONFIG" || { echo "$PRINTSERVER_CONFIG not existing.";
	if [ "$1" = "stop" ]; then exit 0;
	else exit 6; fi; }

test -r "$PRINTSERVER_HOME/fopconfig.xml" || { echo "$PRINTSERVER_HOME/fopconf.xml not existing.";
	if [ "$1" = "stop" ]; then exit 0;
	else exit 6; fi; }

# Run config file
CODEPAGE="$(sed -n '/codepage/{s/^[ \t(<codepage>)]*//;s/[ \t]*<\/.*$//;p}' $PRINTSERVER_CONFIG)"

export PRINTSERVER_HOME
export JAVA_HOME
test $(echo "$PATH" | grep "$PRINTSERVER_HOME") || PATH=$PATH:"$PRINTSERVER_HOME"
test $(echo "$PATH" | grep "$JAVA_HOME") || PATH=$PATH:"$JAVA_HOME/bin"
export PATH

# Check for missing binaries (stale symlinks should not happen)
test -x "$JAVA_HOME/bin/java" || { echo "Java not installed."; 
	if [ "$1" = "stop" ]; then exit 0;
	else exit 5; fi; }
test -f "$PRINTSERVER_HOME/printserver.jar" || { echo "$PRINTSERVER_HOME/printserver.jar not installed."; 
	if [ "$1" = "stop" ]; then exit 0;
	else exit 5; fi; }

# Check for suitable Java version
if ! java -version 2>&1 | head -1 | grep "version" > /dev/null 2>&1
then
	echo "Not able to check Java version."
	exit 1
fi
if java -version 2>&1 | head -1 | grep "1.[0-5]." > /dev/null 2>&1
then
	echo "Invalid Java Runtime Environment. Need version 1.5.x or higher"
	exit 5
fi


case "$1" in
    start)
	echo "Starting Hmedia PrintServer."

#	check for running instances
	if [ $(running_instances) != "0" ]
	then
		echo "Hmedia PrintServer already running. Not started."
		exit 0
	fi

	cd $PRINTSERVER_HOME
	java -Dfile.encoding=$CODEPAGE -jar $PRINTSERVER_HOME/printserver.jar >> /var/log/PrintServer 2>&1 &
	sleep 1
	if [ $(running_instances) == "1" ]
	then
		echo "The PrintServer logfile is /var/log/PrintServer. Watch it for exceptions thrown at startup."
		exit 0
	else
		exit 1
	fi
	;;


    stop)
	echo "Shutting down Hmedia PrintServer."

#	check for a running instance
	case $(running_instances) in
	    "0")
		echo "Hmedia PrintServer is not running."
		exit 0
		;;
	    "1")
		java -cp $PRINTSERVER_HOME/printserver.jar:$PRINTSERVER_HOME hmedia.printmanager.server.PrintServerClient ///PrintServer destroy >> /var/log/PrintServer 2>&1
		sleep 2
		if ps -ef | grep "printserver.jar" | grep -v "grep" > /dev/null 2>$1
		then
			PID=$(ps -ef | grep "printserver.jar" | grep -v "grep" | awk '{print $2}')
			kill -9 $PID
		fi
		if ps -ef | grep "printserver.jar" | grep -v "grep" > /dev/null 2>$1
		then
			echo "Not able to stop Hmedia PrintServer. Stop program manually!"
			exit 1
		fi
		exit 0
		;;
	    *)
		echo "WARNING!"
		echo "More than one inctance of Hmedia PrintServer are running."
		exit 1
		;;
	esac
	;;


    restart)
	## Stop the service and regardless of whether it was
	## running or not, start it again.
	if $0 stop
	then
		$0 start
	else
		echo "Hmedia PrintServer restart failed."
		echo "The running instance could not be stopped."
		exit 1
	fi
	;;


    status)
	echo "Checking for service Hmedia PrintServer."

#	check for a running instance
	case $(running_instances) in
	    "0")
		echo "Hmedia PrintServer is not running."
		exit 3
		;;
	    "1")
		## check status with builtin component, if process is running
		java -cp $PRINTSERVER_HOME/printserver.jar:$PRINTSERVER_HOME hmedia.printmanager.server.PrintServerClient ///PrintServer status
		exit 0
		;;
	    *)
		echo "WARNING!"
		echo "More than one instance of Hmedia PrintServer is running."
		exit 4
		;;
	esac

	# Return value is slightly different for the status command:
	# 0 - service up and running
	# 1 - service dead, but /var/run/  pid  file exists
	# 2 - service dead, but /var/lock/ lock file exists
	# 3 - service not running (unused)
	# 4 - service status unknown :-(
	# 5--199 reserved (5--99 LSB, 100--149 distro, 150--199 appl.)
	;;


    version)
	echo "Checking for Hmedia PrintServer version."

#	check for a running instance
	if ! ps -ef | grep "printserver.jar" | grep -v "grep" > /dev/null 2>$1
	then
		echo "Hmedia PrintServer is not running."
		exit 7
	else
		java -cp $PRINTSERVER_HOME/printserver.jar:$PRINTSERVER_HOME hmedia.printmanager.server.PrintServerClient ///PrintServer version
		exit 0
	fi
	;;
    license)
	echo "Checking for Hmedia PrintServer license."

#	check for a running instance
	if ! ps -ef | grep "printserver.jar" | grep -v "grep" > /dev/null 2>$1
	then
		echo "Hmedia PrintServer is not running."
		exit 7
	else
		java -cp $PRINTSERVER_HOME/printserver.jar:$PRINTSERVER_HOME hmedia.printmanager.server.PrintServerClient ///PrintServer license
		exit 0
	fi
	;;
    xml-output)
	echo "Toggle xmloutput setting for Hmedia PrintServer"

#	check for a running instance
	if ! ps -ef | grep "printserver.jar" | grep -v "grep" > /dev/null 2>$1
	then
		echo "Hmedia PrintServer is not running."
		exit 7
	else
		java -cp $PRINTSERVER_HOME/printserver.jar:$PRINTSERVER_HOME hmedia.printmanager.server.PrintServerClient ///PrintServer xmloutput
		exit 0
	fi
	;;
    *)
	echo "Usage: $0 {start|stop|status|restart|version|license|xml-output}"
	exit 2
	;;
esac
